import {mat4, vec3, quat, mat3} from "gl-matrix";

export class Camera{
    pos:vec3;
    rot:quat;
    #proj : mat4;
    fovY : number;
    aspect : number;
    far : number;
    near: number;
    constructor(pos:vec3,rot?:quat){
        this.pos = pos;
        this.#proj = mat4.create();
        if(rot == undefined){
            const lookAt = mat4.lookAt(mat4.create(),this.pos,[0,0,0],[0,1,0]);
            const rotMat = mat3.fromMat4(mat3.create(),lookAt);
            this.rot = quat.fromMat3(quat.create(), rotMat);
            //quat.normalize(this.rot,this.rot);
            //quat.conjugate(this.rot,this.rot);
        }
        else{
            this.rot = rot;
        }
    }
    rotateRight(rad:number){
        quat.rotateY(this.rot,this.rot,rad);
    }
    rotateUp(rad:number){
        quat.rotateZ(this.rot,this.rot,rad);
    }
    rotateClockwise(rad:number){
        quat.rotateX(this.rot,this.rot,rad);
    }
    get viewMatrix():mat4{
        const p = vec3.negate(vec3.create() ,this.pos);
        const t = mat4.fromTranslation(mat4.create(),p);
        const r = mat4.fromQuat(mat4.create(),this.rot);
        return mat4.mul(mat4.create(),r,t);
    }
    /**
     * update projection matrix
     */
    UpdateProjectionMat(){
        mat4.perspective(this.#proj,this.fovY,this.aspect,this.near,this.far);
    }
    get projectionMatrix():mat4{
        return this.#proj;
    }
};