import {createProgramFromSource, ProgramError, ShaderError} from "./gl_util";

const vert_src = `#version 300 es
layout(location=0) in vec4 pos;

void main() {
  gl_Position = pos;
}
`;
const frag_src = `#version 300 es
precision highp float;
layout(location=0) out vec4 outColor;
//in vec4 gl_FragCoord;
//in vec2 gl_PointCoord;
uniform vec4 u_color;

void main() {
  outColor = u_color;
}`;
import { Drawable, RenderState } from "./drawable";
import * as G from "./glWrapper";
import { getUniformDefaultValue, UniformSet } from "./uniform";

export class Drawer2D implements Drawable{
    gl : WebGL2RenderingContext;
    uniforms : UniformSet;
    program: G.GLProgram;
    
    vao: G.VertexArray;
    positionBuffer: G.VertexBuffer;
    indexBuffer: G.IndexBuffer;
    constructor(gl: WebGL2RenderingContext){
        this.gl = gl;
        this.uniforms = getUniformDefaultValue();
        try{
            this.program = new G.GLProgram(createProgramFromSource(gl,vert_src,frag_src));
            this.program.use(gl);
        }
        catch(e){
            if(e instanceof ShaderError){
                console.log(e.message,"\n",e.info);
            }
            else if(e instanceof ProgramError){
                console.log(e.message,"\n",e.info);
            }
            else throw e;
        }
    }
    prepare(){
        const gl = this.gl;
        const position = [
            -0.5,-0.5,
            0.5,-0.5,
            0.5,0.5,
            -0.5,0.5
        ];
        const index = [
            0,1,2,
            2,3,0
        ];
        this.vao = G.createVertexArray(gl);
        const posLoc = this.program.getAttribLocation(gl,"pos");
        this.vao.bind(gl);
        
        this.positionBuffer = G.createVertexBuffer(gl,position);
        this.vao.addBuffer(gl,this.positionBuffer,posLoc,{
            count:2,
            type:gl.FLOAT,
            normalized:false,
            stride:0,
            offset:0
        });

        this.indexBuffer = G.createIndexBuffer(gl,index);
        this.indexBuffer.bind(gl);
        this.vao.unbind(gl);
        this.setUniform(this.uniforms);
    }
    useProgram(){
        this.program.use(this.gl);
    }
    setUniform(u:UniformSet){
        this.uniforms = u;
    }
    draw(gl:WebGL2RenderingContext, state : RenderState){
        this.useProgram();

        this.program.setUniform4fv(gl,"u_color",[this.uniforms.redcolor,0.3,0.8,1.0]);
        this.vao.bind(gl);

        gl.drawElements(gl.TRIANGLES,this.indexBuffer.count,gl.UNSIGNED_SHORT,0);
        this.vao.unbind(gl);
    }
}